@@ -3,8 +3,10 @@
 """
 import logging
 import warnings
+from pathlib import Path
 from typing import Any, Dict, List, Optional, Tuple, Union
 
+import dask.array as da
 import numpy as np
 import zarr
 
@@ -15,6 +17,9 @@ from .types import JSONDict
 
 LOGGER = logging.getLogger("ome_zarr.writer")
 
+ListOfArrayLike = Union[List[da.Array], List[np.ndarray]]
+ArrayLike = Union[da.Array, np.ndarray]
+
 
 def _get_valid_axes(
     ndim: int = None,
@@ -32,10 +37,10 @@ def _get_valid_axes(
     if axes is None:
         if ndim == 2:
             axes = ["y", "x"]
-            LOGGER.info("Auto using axes %s for 2D data" % axes)
+            LOGGER.info("Auto using axes %s for 2D data", axes)
         elif ndim == 5:
             axes = ["t", "c", "z", "y", "x"]
-            LOGGER.info("Auto using axes %s for 5D data" % axes)
+            LOGGER.info("Auto using axes %s for 5D data", axes)
         else:
             raise ValueError(
                 "axes must be provided. Can't be guessed for 3D or 4D data"
@@ -70,7 +75,7 @@ def _validate_well_images(
             validated_images.append({"path": str(image)})
         elif isinstance(image, dict):
             if any(e not in VALID_KEYS for e in image.keys()):
-                LOGGER.debug("f{image} contains unspecified keys")
+                LOGGER.debug("%s contains unspecified keys", image)
             if "path" not in image:
                 raise ValueError(f"{image} must contain a path key")
             if not isinstance(image["path"], str):
@@ -100,7 +105,7 @@ def _validate_plate_acquisitions(
         if not isinstance(acquisition, dict):
             raise ValueError(f"{acquisition} must be a dictionary")
         if any(e not in VALID_KEYS for e in acquisition.keys()):
-            LOGGER.debug("f{acquisition} contains unspecified keys")
+            LOGGER.debug("%s contains unspecified keys", acquisition)
         if "id" not in acquisition:
             raise ValueError(f"{acquisition} must contain an id key")
         if not isinstance(acquisition["id"], int):
@@ -169,19 +174,20 @@ def _validate_plate_wells(
 
 
 def write_multiscale(
-    pyramid: List,
+    pyramid: ListOfArrayLike,
     group: zarr.Group,
     chunks: Union[Tuple[Any, ...], int] = None,
     fmt: Format = CurrentFormat(),
     axes: Union[str, List[str], List[Dict[str, str]]] = None,
     coordinate_transformations: List[List[Dict[str, Any]]] = None,
     storage_options: Union[JSONDict, List[JSONDict]] = None,
+    name: str = None,
     **metadata: Union[str, JSONDict, List[JSONDict]],
 ) -> None:
     """
     Write a pyramid with multiscale metadata to disk.
 
-    :type pyramid: list of :class:`numpy.ndarray`
+    :type pyramid: list of :class:`numpy.ndarray` or :class:`dask.array.Array`
     :param pyramid:
         The image data to save. Largest level first. All image arrays MUST be up to
         5-dimensional with dimensions ordered (t, c, z, y, x)
@@ -214,7 +220,6 @@ def write_multiscale(
         One can provide different chunk size for each level of a pyramid using this
         option.
     """
-
     dims = len(pyramid[0].shape)
     axes = _get_valid_axes(dims, axes, fmt)
 
@@ -225,13 +230,8 @@ Please use the 'storage_options' argument instead."""
 
     datasets: List[dict] = []
     for path, data in enumerate(pyramid):
-        options = {}
-        if storage_options:
-            options = (
-                storage_options
-                if not isinstance(storage_options, list)
-                else storage_options[path]
-            )
+        options = _resolve_storage_options(storage_options, path)
+
         # ensure that the chunk dimensions match the image dimensions
         # (which might have been changed for versions 0.1 or 0.2)
         # if chunks are explicitly set in the storage options
@@ -240,7 +240,21 @@ Please use the 'storage_options' argument instead."""
         # chunks_opt = options.pop("chunks", None)
         if chunks_opt is not None:
             chunks_opt = _retuple(chunks_opt, data.shape)
-        group.create_dataset(str(path), data=data, chunks=chunks_opt, **options)
+
+        if isinstance(data, da.Array):
+            if chunks_opt is not None:
+                data = da.array(data).rechunk(chunks=chunks_opt)
+                options["chunks"] = chunks_opt
+            da.to_zarr(
+                arr=data,
+                url=group.store,
+                component=str(Path(group.path, str(path))),
+                storage_options=options,
+                compressor=options.get("compressor"),
+            )
+        else:
+            group.create_dataset(str(path), data=data, chunks=chunks_opt, **options)
+
         datasets.append({"path": str(path)})
 
     if coordinate_transformations is None:
@@ -255,7 +269,7 @@ Please use the 'storage_options' argument instead."""
         for dataset, transform in zip(datasets, coordinate_transformations):
             dataset["coordinateTransformations"] = transform
 
-    write_multiscales_metadata(group, datasets, fmt, axes, **metadata)
+    write_multiscales_metadata(group, datasets, fmt, axes, name, **metadata)
 
 
 def write_multiscales_metadata(
@@ -263,6 +277,7 @@ def write_multiscales_metadata(
     datasets: List[dict],
     fmt: Format = CurrentFormat(),
     axes: Union[str, List[str], List[Dict[str, str]]] = None,
+    name: str = None,
     **metadata: Union[str, JSONDict, List[JSONDict]],
 ) -> None:
     """
@@ -302,6 +317,7 @@ def write_multiscales_metadata(
         dict(
             version=fmt.version,
             datasets=_validate_datasets(datasets, ndim, fmt),
+            name=name if name else group.name,
             **metadata,
         )
     ]
@@ -385,7 +401,7 @@ def write_well_metadata(
 
 
 def write_image(
-    image: np.ndarray,
+    image: ArrayLike,
     group: zarr.Group,
     scaler: Scaler = Scaler(),
     chunks: Union[Tuple[Any, ...], int] = None,
@@ -397,12 +413,12 @@ def write_image(
 ) -> None:
     """Writes an image to the zarr store according to ome-zarr specification
 
-    :type image: :class:`numpy.ndarray`
+    :type image: :class:`numpy.ndarray` or `dask.array.Array`
     :param image:
       The image data to save. A downsampling of the data will be computed
       if the scaler argument is non-None.
       Image array MUST be up to 5-dimensional with dimensions
-      ordered (t, c, z, y, x)
+      ordered (t, c, z, y, x).  Image can be a numpy or dask Array.
     :type group: :class:`zarr.hierarchy.Group`
     :param group: The group within the zarr store to write the metadata in.
     :type scaler: :class:`ome_zarr.scale.Scaler`
@@ -435,17 +451,132 @@ def write_image(
         One can provide different chunk size for each level of a pyramid using this
         option.
     """
-    mip, axes = _create_mip(image, fmt, scaler, axes)
-    write_multiscale(
-        mip,
-        group,
-        chunks=chunks,
-        fmt=fmt,
-        axes=axes,
-        coordinate_transformations=coordinate_transformations,
-        storage_options=storage_options,
-        **metadata,
+    if isinstance(image, da.Array):
+        _write_dask_image(
+            image,
+            group,
+            scaler,
+            chunks=chunks,
+            fmt=fmt,
+            axes=axes,
+            coordinate_transformations=coordinate_transformations,
+            storage_options=storage_options,
+            name=None,
+            **metadata,
+        )
+    else:
+        mip, axes = _create_mip(image, fmt, scaler, axes)
+        write_multiscale(
+            mip,
+            group,
+            chunks=chunks,
+            fmt=fmt,
+            axes=axes,
+            coordinate_transformations=coordinate_transformations,
+            storage_options=storage_options,
+            name=None,
+            **metadata,
+        )
+
+
+def _resolve_storage_options(
+    storage_options: Union[JSONDict, List[JSONDict], None], path: int
+) -> JSONDict:
+    options = {}
+    if storage_options:
+        options = (
+            storage_options.copy()
+            if not isinstance(storage_options, list)
+            else storage_options[path]
+        )
+    return options
+
+
+def _write_dask_image(
+    image: da.Array,
+    group: zarr.Group,
+    scaler: Scaler = Scaler(),
+    chunks: Union[Tuple[Any, ...], int] = None,
+    fmt: Format = CurrentFormat(),
+    axes: Union[str, List[str], List[Dict[str, str]]] = None,
+    coordinate_transformations: List[List[Dict[str, Any]]] = None,
+    storage_options: Union[JSONDict, List[JSONDict]] = None,
+    name: str = None,
+    **metadata: Union[str, JSONDict, List[JSONDict]],
+) -> None:
+
+    if fmt.version in ("0.1", "0.2"):
+        # v0.1 and v0.2 are strictly 5D
+        shape_5d: Tuple[Any, ...] = (*(1,) * (5 - image.ndim), *image.shape)
+        image = image.reshape(shape_5d)
+        # and we don't need axes
+        axes = None
+
+    dims = len(image.shape)
+    axes = _get_valid_axes(dims, axes, fmt)
+
+    if chunks is not None:
+        msg = """The 'chunks' argument is deprecated and will be removed in version 0.5.
+Please use the 'storage_options' argument instead."""
+        warnings.warn(msg, DeprecationWarning)
+
+    datasets: List[dict] = []
+    delayed = []
+
+    # for path, data in enumerate(pyramid):
+    max_layer: int = scaler.max_layer if scaler is not None else 0
+    shapes = []
+    for path in range(0, max_layer + 1):
+        # LOGGER.debug(f"write_image path: {path}")
+        options = _resolve_storage_options(storage_options, path)
+
+        # don't downsample top level of pyramid
+        if str(path) != "0" and scaler is not None:
+            image = scaler.resize_image(image)
+
+        # ensure that the chunk dimensions match the image dimensions
+        # (which might have been changed for versions 0.1 or 0.2)
+        # if chunks are explicitly set in the storage options
+        chunks_opt = options.pop("chunks", chunks)
+        # switch to this code in 0.5
+        # chunks_opt = options.pop("chunks", None)
+        if chunks_opt is not None:
+            chunks_opt = _retuple(chunks_opt, image.shape)
+            image = da.array(image).rechunk(chunks=chunks_opt)
+            options["chunks"] = chunks_opt
+        LOGGER.debug("chunks_opt: %s", chunks_opt)
+        shapes.append(image.shape)
+
+        LOGGER.debug(
+            "write dask.array to_zarr shape: %s, dtype: %s", image.shape, image.dtype
+        )
+        delayed.append(
+            da.to_zarr(
+                arr=image,
+                url=group.store,
+                component=str(Path(group.path, str(path))),
+                storage_options=options,
+                compute=False,
+                compressor=options.get("compressor"),
+            )
+        )
+        datasets.append({"path": str(path)})
+
+    da.compute(*delayed)
+
+    if coordinate_transformations is None:
+        # shapes = [data.shape for data in delayed]
+        coordinate_transformations = fmt.generate_coordinate_transformations(shapes)
+
+    # we validate again later, but this catches length mismatch before zip(datasets...)
+    fmt.validate_coordinate_transformations(
+        dims, len(datasets), coordinate_transformations
     )
+    if coordinate_transformations is not None:
+        for dataset, transform in zip(datasets, coordinate_transformations):
+            dataset["coordinateTransformations"] = transform
+
+    write_multiscales_metadata(group, datasets, fmt, axes, name, **metadata)
 
 
 def write_label_metadata(
@@ -573,7 +704,7 @@ def write_multiscale_labels(
 
 
 def write_labels(
-    labels: np.ndarray,
+    labels: Union[np.ndarray, da.Array],
     group: zarr.Group,
     name: str,
     scaler: Scaler = Scaler(),
@@ -634,18 +765,38 @@ def write_labels(
     :param label_metadata:
       Image label metadata. See :meth:`write_label_metadata` for details
     """
-    mip, axes = _create_mip(labels, fmt, scaler, axes)
-    write_multiscale_labels(
-        mip,
-        group,
+    sub_group = group.require_group(f"labels/{name}")
+    if isinstance(labels, da.Array):
+        _write_dask_image(
+            labels,
+            sub_group,
+            scaler,
+            chunks=chunks,
+            fmt=fmt,
+            axes=axes,
+            coordinate_transformations=coordinate_transformations,
+            storage_options=storage_options,
+            name=name,
+            **metadata,
+        )
+    else:
+        mip, axes = _create_mip(labels, fmt, scaler, axes)
+        write_multiscale(
+            mip,
+            sub_group,
+            chunks=chunks,
+            fmt=fmt,
+            axes=axes,
+            coordinate_transformations=coordinate_transformations,
+            storage_options=storage_options,
+            name=name,
+            **metadata,
+        )
+    write_label_metadata(
+        group=group["labels"],
         name=name,
-        chunks=chunks,
         fmt=fmt,
-        axes=axes,
-        coordinate_transformations=coordinate_transformations,
-        storage_options=storage_options,
-        label_metadata=label_metadata,
-        **metadata,
+        **({} if label_metadata is None else label_metadata),
     )
 
 
@@ -684,6 +835,12 @@ def _create_mip(
 def _retuple(
     chunks: Union[Tuple[Any, ...], int], shape: Tuple[Any, ...]
 ) -> Tuple[Any, ...]:
+    """
+    Expand chunks to match shape.
+
+    E.g. if chunks is (64, 64) and shape is (3, 4, 5, 1028, 1028)
+    return (3, 4, 5, 64, 64)
+    """
 
     _chunks: Tuple[Any, ...]
     if isinstance(chunks, int):
